@@ -79,21 +79,58 @@
  *     a SEGMENT_START have posted a SEGMENT_DONE.</para></listitem>
  *   </varlistentry>
  *   <varlistentry>
+ *     <term>GST_MESSAGE_DURATION</term>
+ *     <listitem><para> Is posted by an element that detected a change
+ *     in the stream duration. The default bin behaviour is to clear any
+ *     cached duration values so that the next duration query will perform
+ *     a full duration recalculation. The duration change is posted to the
+ *     application so that it can refetch the new duration with a duration
+ *     query.
+ *     </para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>GST_MESSAGE_CLOCK_LOST</term>
+ *     <listitem><para> This message is posted by an element when it
+ *     can no longer provide a clock. The default bin behaviour is to
+ *     check if the lost clock was the one provided by the bin. If so and
+ *     the bin is currently in the PLAYING state, the message is forwarded to
+ *     the bin parent.
+ *     This message is also generated when a clock provider is removed from
+ *     the bin. If this message is received by the application, it should
+ *     PAUSE the pipeline and set it back to PLAYING to force a new clock
+ *     distribution.
+ *     </para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
+ *     <term>GST_MESSAGE_CLOCK_PROVIDE</term>
+ *     <listitem><para> This message is generated when an element
+ *     can provide a clock. This mostly happens when a new clock 
+ *     provider is added to the bin. The default behaviour of the bin is to 
+ *     mark the currently selected clock as dirty, which will perform a clock
+ *     recalculation the next time the bin is asked to provide a clock.
+ *     This message is never sent tot the application but is forwarded to
+ *     the parent of the bin.
+ *     </para></listitem>
+ *   </varlistentry>
+ *   <varlistentry>
  *     <term>OTHERS</term>
  *     <listitem><para> posted upwards.</para></listitem>
  *   </varlistentry>
  * </variablelist>
  *
+ *
  * A #GstBin implements the following default behaviour for answering to a
  * #GstQuery:
  * <variablelist>
  *   <varlistentry>
  *     <term>GST_QUERY_DURATION</term>
- *     <listitem><para>If the query has been asked before with the same format,
+ *     <listitem><para>If the query has been asked before with the same format
+ *     and the bin is a toplevel bin (ie. has no parent),
  *     use the cached previous value. If no previous value was cached, the
  *     query is sent to all sink elements in the bin and the MAXIMUM of all
- *     values is returned and cached. If no sinks are available in the bin, the
- *     query fails.</para></listitem>
+ *     values is returned. If the bin is a toplevel bin the value is cached. 
+ *     If no sinks are available in the bin, the query fails.
+ *     </para></listitem>
  *   </varlistentry>
  *   <varlistentry>
  *     <term>OTHERS</term>
@@ -110,7 +147,7 @@
  * </para>
  * </refsect2>
  *
- * Last reviewed on 2005-12-16 (0.10.0)
+ * Last reviewed on 2006-03-12 (0.10.5)
  */
 
 #include "gst_private.h"
@@ -380,14 +417,17 @@ static void
 gst_bin_dispose (GObject * object)
 {
   GstBin *bin = GST_BIN (object);
+  GstBus **child_bus_p = &bin->child_bus;
+  GstClock **provided_clock_p = &bin->provided_clock;
+  GstElement **clock_provider_p = &bin->clock_provider;
 
   GST_CAT_DEBUG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
 
   bin_remove_messages (bin, NULL, GST_MESSAGE_ANY);
 
-  gst_object_replace ((GstObject **) & bin->child_bus, NULL);
-  gst_object_replace ((GstObject **) & bin->provided_clock, NULL);
-  gst_object_replace ((GstObject **) & bin->clock_provider, NULL);
+  gst_object_replace ((GstObject **) child_bus_p, NULL);
+  gst_object_replace ((GstObject **) provided_clock_p, NULL);
+  gst_object_replace ((GstObject **) clock_provider_p, NULL);
 
   while (bin->children) {
     gst_bin_remove (bin, GST_ELEMENT_CAST (bin->children->data));
@@ -480,6 +520,8 @@ gst_bin_provide_clock_func (GstElement * element)
   GstBin *bin;
   GstIterator *it;
   gpointer val;
+  GstClock **provided_clock_p;
+  GstElement **clock_provider_p;
 
   bin = GST_BIN (element);
 
@@ -509,10 +551,11 @@ gst_bin_provide_clock_func (GstElement * element)
       gst_object_unref (child);
     }
   }
-  gst_object_replace ((GstObject **) & bin->provided_clock,
-      (GstObject *) result);
-  gst_object_replace ((GstObject **) & bin->clock_provider,
-      (GstObject *) provider);
+
+  provided_clock_p = &bin->provided_clock;
+  clock_provider_p = &bin->clock_provider;
+  gst_object_replace ((GstObject **) provided_clock_p, (GstObject *) result);
+  gst_object_replace ((GstObject **) clock_provider_p, (GstObject *) provider);
   bin->clock_dirty = FALSE;
   GST_DEBUG_OBJECT (bin,
       "provided new clock %" GST_PTR_FORMAT " by provider %" GST_PTR_FORMAT,
@@ -1918,6 +1961,21 @@ gst_bin_recalc_func (GstBin * bin, gpointer data)
   gst_object_unref (bin);
 }
 
+static GstBusSyncReply
+bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
+{
+
+  GstBinClass *bclass;
+
+  bclass = GST_BIN_GET_CLASS (bin);
+  if (bclass->handle_message)
+    bclass->handle_message (bin, message);
+  else
+    gst_message_unref (message);
+
+  return GST_BUS_DROP;
+}
+
 /* handle child messages:
  *
  * GST_MESSAGE_EOS: This message is only posted by sinks
@@ -1941,23 +1999,26 @@ gst_bin_recalc_func (GstBin * bin, gpointer data)
  *     changes its duration marks our cached values invalid.
  *     This message is also posted upwards.
  *
+ * GST_MESSAGE_CLOCK_LOST: This message is posted by an element when it
+ *     can no longer provide a clock. The default bin behaviour is to
+ *     check if the lost clock was the one provided by the bin. If so and
+ *     we are currently in the PLAYING state, we forward the message to 
+ *     our parent.
+ *     This message is also generated when we remove a clock provider from
+ *     a bin. If this message is received by the application, it should
+ *     PAUSE the pipeline and set it back to PLAYING to force a new clock
+ *     distribution.
+ *
+ * GST_MESSAGE_CLOCK_PROVIDE: This message is generated when an element
+ *     can provide a clock. This mostly happens when we add a new clock
+ *     provider to the bin. The default behaviour of the bin is to mark the
+ *     currently selected clock as dirty, which will perform a clock
+ *     recalculation the next time we are asked to provide a clock.
+ *     This message is never sent tot the application but is forwarded to
+ *     the parent.
+ *
  * OTHER: post upwards.
  */
-static GstBusSyncReply
-bin_bus_handler (GstBus * bus, GstMessage * message, GstBin * bin)
-{
-
-  GstBinClass *bclass;
-
-  bclass = GST_BIN_GET_CLASS (bin);
-  if (bclass->handle_message)
-    bclass->handle_message (bin, message);
-  else
-    gst_message_unref (message);
-
-  return GST_BUS_DROP;
-}
-
 static void
 gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
 {
@@ -2076,6 +2137,8 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
     }
     case GST_MESSAGE_CLOCK_LOST:
     {
+      GstClock **provided_clock_p;
+      GstElement **clock_provider_p;
       gboolean playing, provided, forward;
       GstClock *clock;
 
@@ -2092,8 +2155,10 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
         GST_DEBUG_OBJECT (bin,
             "Lost clock %" GST_PTR_FORMAT " provided by %" GST_PTR_FORMAT,
             bin->provided_clock, bin->clock_provider);
-        gst_object_replace ((GstObject **) & bin->provided_clock, NULL);
-        gst_object_replace ((GstObject **) & bin->clock_provider, NULL);
+        provided_clock_p = &bin->provided_clock;
+        clock_provider_p = &bin->clock_provider;
+        gst_object_replace ((GstObject **) provided_clock_p, NULL);
+        gst_object_replace ((GstObject **) clock_provider_p, NULL);
       }
       GST_DEBUG_OBJECT (bin, "provided %d, playing %d, forward %d",
           provided, playing, forward);
@@ -2148,10 +2213,10 @@ typedef struct
 typedef void (*QueryInitFunction) (GstBin * bin, QueryFold * fold);
 typedef void (*QueryDoneFunction) (GstBin * bin, QueryFold * fold);
 
-/* for duration we collect all durations and take the MAX of
- * all valid results */
+/* for duration/position we collect all durations/positions and take 
+ * the MAX of all valid results */
 static void
-bin_query_duration_init (GstBin * bin, QueryFold * fold)
+bin_query_max_init (GstBin * bin, QueryFold * fold)
 {
   fold->max = -1;
 }
@@ -2195,6 +2260,37 @@ bin_query_duration_done (GstBin * bin, QueryFold * fold)
 #endif
 }
 
+static gboolean
+bin_query_position_fold (GstElement * item, GValue * ret, QueryFold * fold)
+{
+  if (gst_element_query (item, fold->query)) {
+    gint64 position;
+
+    g_value_set_boolean (ret, TRUE);
+
+    gst_query_parse_position (fold->query, NULL, &position);
+
+    GST_DEBUG_OBJECT (item, "got position %" G_GINT64_FORMAT, position);
+
+    if (position > fold->max)
+      fold->max = position;
+  }
+
+  gst_object_unref (item);
+  return TRUE;
+}
+static void
+bin_query_position_done (GstBin * bin, QueryFold * fold)
+{
+  GstFormat format;
+
+  gst_query_parse_position (fold->query, &format, NULL);
+  /* store max in query result */
+  gst_query_set_position (fold->query, format, fold->max);
+
+  GST_DEBUG_OBJECT (bin, "max position %" G_GINT64_FORMAT, fold->max);
+}
+
 /* generic fold, return first valid result */
 static gboolean
 bin_query_generic_fold (GstElement * item, GValue * ret, QueryFold * fold)
@@ -2260,10 +2356,17 @@ gst_bin_query (GstElement * element, GstQuery * query)
       GST_OBJECT_UNLOCK (bin);
 #endif
       fold_func = (GstIteratorFoldFunction) bin_query_duration_fold;
-      fold_init = bin_query_duration_init;
+      fold_init = bin_query_max_init;
       fold_done = bin_query_duration_done;
       break;
     }
+    case GST_QUERY_POSITION:
+    {
+      fold_func = (GstIteratorFoldFunction) bin_query_position_fold;
+      fold_init = bin_query_max_init;
+      fold_done = bin_query_position_done;
+      break;
+    }
     default:
       fold_func = (GstIteratorFoldFunction) bin_query_generic_fold;
       break;
