@@ -20,6 +20,34 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstplugin
+ * @short_description: Container for features loaded from a shared object module
+ * @see_also: #GstPluginFeature, #GstElementFactory
+ *
+ * GStreamer is extensible, so #GstElement instances can be loaded at runtime.
+ * A plugin system can provide one or more of the basic
+ * <application>GStreamer</application> #GstPluginFeature subclasses.
+ *
+ * A plugin should export a symbol <symbol>plugin_desc</symbol> that is a
+ * struct of type #GstPluginDesc.
+ * the plugin loader will check the version of the core library the plugin was
+ * linked against and will create a new #GstPlugin. It will then call the
+ * #GstPluginInitFunc function that was provided in the plugin_desc.
+ *
+ * Once you have a handle to a #GstPlugin (e.g. from the #GstRegistryPool), you
+ * can add any object that subclasses #GstPluginFeature.
+ *
+ * Use gst_plugin_find_feature() and gst_plugin_get_feature_list() to find
+ * features in a plugin.
+ *
+ * Usually plugins are always automaticlly loaded so you don't need to call
+ * gst_plugin_load() explicitly to bring it into memory. There are options to
+ * statically link plugins to an app or even use GStreamer without a plugin
+ * repository in which case gst_plugin_load() can be needed to bring the plugin
+ * into memory.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -38,9 +66,10 @@
 
 #include "gstplugin.h"
 #include "gstversion.h"
-#include "gstregistrypool.h"
 #include "gstinfo.h"
 #include "gstfilter.h"
+#include "gstregistry.h"
+#include "gstmacros.h"
 
 
 #define GST_CAT_DEFAULT GST_CAT_PLUGIN_LOADING
@@ -64,39 +93,58 @@ static gboolean _gst_plugin_fault_handler_is_setup = FALSE;
  * GPL: http://www.gnu.org/copyleft/gpl.html
  * LGPL: http://www.gnu.org/copyleft/lesser.html
  * QPL: http://www.trolltech.com/licenses/qpl.html
+ * MPL: http://www.opensource.org/licenses/mozilla1.1.php
  */
-static gchar *valid_licenses[] = {
+static const gchar *valid_licenses[] = {
   "LGPL",                       /* GNU Lesser General Public License */
   "GPL",                        /* GNU General Public License */
   "QPL",                        /* Trolltech Qt Public License */
   "GPL/QPL",                    /* Combi-license of GPL + QPL */
+  "MPL",                        /* MPL 1.1 license */
+  "Proprietary",                /* Proprietary license */
   GST_LICENSE_UNKNOWN,          /* some other license */
   NULL
 };
 
-static void gst_plugin_desc_copy (GstPluginDesc * dest,
-    const GstPluginDesc * src);
-
 static GstPlugin *gst_plugin_register_func (GstPlugin * plugin,
     GModule * module, GstPluginDesc * desc);
+static void
+gst_plugin_desc_copy (GstPluginDesc * dest, const GstPluginDesc * src);
+static void gst_plugin_desc_free (GstPluginDesc * desc);
 
-static GstPlugin *
-gst_plugin_copy (GstPlugin * plugin)
+
+G_DEFINE_TYPE (GstPlugin, gst_plugin, GST_TYPE_OBJECT);
+
+static void
+gst_plugin_init (GstPlugin * plugin)
 {
-  return g_memdup (plugin, sizeof (*plugin));
+
 }
 
-GType
-gst_plugin_get_type (void)
+static void
+gst_plugin_finalize (GObject * object)
 {
-  static GType plugin_type;
+  GstPlugin *plugin = GST_PLUGIN (object);
+  GstRegistry *registry = gst_registry_get_default ();
+  GList *g;
 
-  if (plugin_type == 0) {
-    plugin_type = g_boxed_type_register_static ("GstPlugin",
-        (GBoxedCopyFunc) gst_plugin_copy, g_free);
+  GST_DEBUG ("finalizing plugin %p", plugin);
+  for (g = registry->plugins; g; g = g->next) {
+    if (g->data == (gpointer) plugin) {
+      g_warning ("removing plugin that is still in registry");
+    }
   }
+  g_free (plugin->filename);
+  g_free (plugin->basename);
+  gst_plugin_desc_free (&plugin->desc);
 
-  return plugin_type;
+  G_OBJECT_CLASS (gst_plugin_parent_class)->finalize (object);
+}
+
+static void
+gst_plugin_class_init (GstPluginClass * klass)
+{
+  G_OBJECT_CLASS (klass)->finalize = GST_DEBUG_FUNCPTR (gst_plugin_finalize);
 }
 
 GQuark
@@ -110,11 +158,11 @@ gst_plugin_error_quark (void)
 }
 
 /* this function can be called in the GCC constructor extension, before
- * the _gst_plugin_initialize() was called. In that case, we store the 
+ * the _gst_plugin_initialize() was called. In that case, we store the
  * plugin description in a list to initialize it when we open the main
  * module later on.
  * When the main module is known, we can register the plugin right away.
- * */
+ */
 void
 _gst_plugin_register_static (GstPluginDesc * desc)
 {
@@ -128,11 +176,11 @@ _gst_plugin_register_static (GstPluginDesc * desc)
 
     if (GST_CAT_DEFAULT)
       GST_LOG ("attempting to load static plugin \"%s\" now...", desc->name);
-    plugin = g_new0 (GstPlugin, 1);
+    plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
     if (gst_plugin_register_func (plugin, main_module, desc)) {
       if (GST_CAT_DEFAULT)
         GST_INFO ("loaded static plugin \"%s\"", desc->name);
-      gst_registry_pool_add_plugin (plugin);
+      gst_default_registry_add_plugin (plugin);
     }
   }
 }
@@ -148,14 +196,13 @@ _gst_plugin_initialize (void)
 }
 
 /* this function could be extended to check if the plugin license matches the
- * application's license
- * (would require the app to register its license somehow).
+ * applications license (would require the app to register its license somehow).
  * We'll wait for someone who's interested in it to code it :)
  */
 static gboolean
 gst_plugin_check_license (const gchar * license)
 {
-  gchar **check_license = valid_licenses;
+  const gchar **check_license = valid_licenses;
 
   g_assert (check_license);
 
@@ -182,25 +229,24 @@ static GstPlugin *
 gst_plugin_register_func (GstPlugin * plugin, GModule * module,
     GstPluginDesc * desc)
 {
-  g_assert (plugin->module == NULL);
-
   if (!gst_plugin_check_version (desc->major_version, desc->minor_version)) {
     if (GST_CAT_DEFAULT)
-      GST_INFO ("plugin \"%s\" has incompatible version, not loading",
+      GST_WARNING ("plugin \"%s\" has incompatible version, not loading",
           plugin->filename);
     return NULL;
   }
 
-  if (!desc->license || !desc->description || !desc->package || !desc->origin) {
+  if (!desc->license || !desc->description || !desc->source ||
+      !desc->package || !desc->origin) {
     if (GST_CAT_DEFAULT)
-      GST_INFO ("plugin \"%s\" has incorrect GstPluginDesc, not loading",
+      GST_WARNING ("plugin \"%s\" has incorrect GstPluginDesc, not loading",
           plugin->filename);
     return NULL;
   }
 
   if (!gst_plugin_check_license (desc->license)) {
     if (GST_CAT_DEFAULT)
-      GST_INFO ("plugin \"%s\" has invalid license \"%s\", not loading",
+      GST_WARNING ("plugin \"%s\" has invalid license \"%s\", not loading",
           plugin->filename, desc->license);
     return NULL;
   }
@@ -209,11 +255,10 @@ gst_plugin_register_func (GstPlugin * plugin, GModule * module,
     GST_LOG ("plugin \"%s\" looks good", GST_STR_NULL (plugin->filename));
 
   gst_plugin_desc_copy (&plugin->desc, desc);
-  plugin->module = module;
 
   if (!((desc->plugin_init) (plugin))) {
     if (GST_CAT_DEFAULT)
-      GST_INFO ("plugin \"%s\" failed to initialise", plugin->filename);
+      GST_WARNING ("plugin \"%s\" failed to initialise", plugin->filename);
     plugin->module = NULL;
     return NULL;
   }
@@ -301,157 +346,95 @@ _gst_plugin_fault_handler_setup (void)
 
 static void _gst_plugin_fault_handler_setup ();
 
-/**
- * gst_plugin_check_module:
- * @module: GModule handle to check for pluginness
- * @error: pointer to a NULL-valued GError
- * @pptr: pointer to a gpointer used to return the gst_plugin_desc symbol
- *        (can be NULL)
- *
- * Checks if the given module is a GStreamer plugin
- *
- * Returns: TRUE if the given module is a GStreamer plugin
- */
-static gboolean
-gst_plugin_check_module (GModule *module, const char *filename, GError **error, gpointer *pptr)
-{
-  gpointer ptr;
-
-  if (pptr == NULL)
-    pptr = &ptr;
-
-  if (module == NULL) {
-    GST_DEBUG ("Error loading plugin %s, reason: %s", filename,
-        g_module_error ());
-    g_set_error (error, GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_MODULE,
-        "Error loading plugin %s, reason: %s", filename, g_module_error ());
-    return FALSE;
-  }
-
-  if (!g_module_symbol (module, "gst_plugin_desc", pptr)) {
-    GST_DEBUG ("Could not find plugin entry point in \"%s\"", filename);
-    g_set_error (error,
-        GST_PLUGIN_ERROR,
-        GST_PLUGIN_ERROR_MODULE,
-        "Could not find plugin entry point in \"%s\"", filename);
-    return FALSE;
-  }
-
-  return TRUE;
-}
-
-/**
- * gst_plugin_check_file:
- * @filename: the plugin filename to check for pluginness
- * @error: pointer to a NULL-valued GError
- *
- * Checks if the given path represents a GStreamer plugin.
- *
- * Returns: TRUE if the given path is a GStreamer plugin.
- */
-gboolean
-gst_plugin_check_file (const gchar * filename, GError ** error)
-{
-  GModule *module;
-  struct stat file_status;
-  gboolean check;
-
-  g_return_val_if_fail (filename != NULL, FALSE);
-
-  if (g_module_supported () == FALSE) {
-    g_set_error (error,
-        GST_PLUGIN_ERROR,
-        GST_PLUGIN_ERROR_MODULE, "Dynamic loading not supported");
-    return FALSE;
-  }
-
-  if (stat (filename, &file_status)) {
-    g_set_error (error,
-        GST_PLUGIN_ERROR,
-        GST_PLUGIN_ERROR_MODULE, "Problem accessing file %s: %s\n", filename,
-        strerror (errno));
-    return FALSE;
-  }
-
-  module = g_module_open (filename, G_MODULE_BIND_LAZY);
-  check = gst_plugin_check_module (module, filename, error, NULL);
-  g_module_close (module);
-
-  GST_INFO ("file \"%s\" %s look like a gst plugin", filename, check ? "does" : "doesn't");
-  return check;
-}
+GStaticMutex gst_plugin_loading_mutex = G_STATIC_MUTEX_INIT;
 
 /**
  * gst_plugin_load_file:
  * @filename: the plugin filename to load
  * @error: pointer to a NULL-valued GError
  *
- * Loads the given plugin.
+ * Loads the given plugin and refs it.  Caller needs to unref after use.
  *
- * Returns: a new GstPlugin or NULL, if an error occurred.
+ * Returns: a reference to the existing loaded GstPlugin, a reference to the
+ * newly-loaded GstPlugin, or NULL if an error occurred.
  */
 GstPlugin *
 gst_plugin_load_file (const gchar * filename, GError ** error)
 {
   GstPlugin *plugin;
   GModule *module;
-  GstPluginDesc *desc;
-  gboolean free_plugin;
+  gboolean ret;
   gpointer ptr;
+  struct stat file_status;
+  GstRegistry *registry;
 
   g_return_val_if_fail (filename != NULL, NULL);
 
+  registry = gst_registry_get_default ();
+  g_static_mutex_lock (&gst_plugin_loading_mutex);
+
+  plugin = gst_registry_lookup (registry, filename);
+  if (plugin) {
+    if (plugin->module) {
+      g_static_mutex_unlock (&gst_plugin_loading_mutex);
+      return plugin;
+    } else {
+      gst_object_unref (plugin);
+      plugin = NULL;
+    }
+  }
+
   GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "attempt to load plugin \"%s\"",
       filename);
 
-
-  module = g_module_open (filename, G_MODULE_BIND_LAZY);
-
-  if (!gst_plugin_check_module (module, filename, error, &ptr))	/* handles module == NULL case */
-    return NULL;
-
-  desc = (GstPluginDesc *) ptr;
-
-  plugin = gst_registry_pool_find_plugin (desc->name);
-  if (!plugin) {
-    free_plugin = TRUE;
-    plugin = g_new0 (GstPlugin, 1);
-    plugin->filename = g_strdup (filename);
-    GST_DEBUG ("created new GstPlugin %p for file \"%s\"", plugin, filename);
-  } else {
-    free_plugin = FALSE;
-    if (gst_plugin_is_loaded (plugin)) {
-      g_module_close (module);
-      if (plugin->filename && strcmp (plugin->filename, filename) != 0) {
-        GST_WARNING
-            ("plugin %p from file \"%s\" with same name %s is already "
-            "loaded, aborting loading of \"%s\"", plugin, plugin->filename,
-            plugin->desc.name, filename);
-        g_set_error (error, GST_PLUGIN_ERROR,
-            GST_PLUGIN_ERROR_NAME_MISMATCH,
-            "plugin %p from file \"%s\" with same name %s is already "
-            "loaded, aborting loading of \"%s\"", plugin, plugin->filename,
-            plugin->desc.name, filename);
-        if (free_plugin)
-          g_free (plugin);
-        return NULL;
-      }
-      GST_LOG ("Plugin %p for file \"%s\" already loaded, returning it now",
-          plugin, filename);
-      return plugin;
-    }
+  if (g_module_supported () == FALSE) {
+    GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "module loading not supported");
+    g_set_error (error,
+        GST_PLUGIN_ERROR,
+        GST_PLUGIN_ERROR_MODULE, "Dynamic loading not supported");
+    goto return_error;
   }
+
+  if (stat (filename, &file_status)) {
+    GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, "problem accessing file");
+    g_set_error (error,
+        GST_PLUGIN_ERROR,
+        GST_PLUGIN_ERROR_MODULE, "Problem accessing file %s: %s\n", filename,
+        strerror (errno));
+    goto return_error;
+  }
+
+  module = g_module_open (filename, G_MODULE_BIND_LOCAL);
+  if (module == NULL) {
+    GST_CAT_WARNING (GST_CAT_PLUGIN_LOADING, "module_open failed: %s",
+        g_module_error ());
+    g_set_error (error,
+        GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_MODULE, "Opening module failed");
+    goto return_error;
+  }
+
+  plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
+
+  plugin->module = module;
+  plugin->filename = g_strdup (filename);
+  plugin->basename = g_path_get_basename (filename);
+  plugin->file_mtime = file_status.st_mtime;
+  plugin->file_size = file_status.st_size;
+
+  ret = g_module_symbol (module, "gst_plugin_desc", &ptr);
+  if (!ret) {
+    GST_DEBUG ("Could not find plugin entry point in \"%s\"", filename);
+    g_set_error (error,
+        GST_PLUGIN_ERROR,
+        GST_PLUGIN_ERROR_MODULE,
+        "File \"%s\" is not a GStreamer plugin", filename);
+    goto return_error;
+  }
+  plugin->orig_desc = (GstPluginDesc *) ptr;
+
   GST_LOG ("Plugin %p for file \"%s\" prepared, calling entry function...",
       plugin, filename);
 
-  if (g_module_symbol (module, "plugin_init", &ptr)) {
-    GST_WARNING
-        ("plugin %p from file \"%s\" exports a symbol named plugin_init\n",
-        plugin, plugin->filename);
-    g_set_error (error, GST_PLUGIN_ERROR, GST_PLUGIN_ERROR_NAME_MISMATCH,
-        "plugin \"%s\" exports a symbol named plugin_init", desc->name);
-  }
-
   /* this is where we load the actual .so, so let's trap SIGSEGV */
   _gst_plugin_fault_handler_setup ();
   _gst_plugin_fault_handler_filename = plugin->filename;
@@ -459,13 +442,7 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
   GST_LOG ("Plugin %p for file \"%s\" prepared, registering...",
       plugin, filename);
 
-  if (gst_plugin_register_func (plugin, module, desc)) {
-    /* remove signal handler */
-    _gst_plugin_fault_handler_restore ();
-    _gst_plugin_fault_handler_filename = NULL;
-    GST_INFO ("plugin \"%s\" loaded", plugin->filename);
-    return plugin;
-  } else {
+  if (!gst_plugin_register_func (plugin, module, plugin->orig_desc)) {
     /* remove signal handler */
     _gst_plugin_fault_handler_restore ();
     GST_DEBUG ("gst_plugin_register_func failed for plugin \"%s\"", filename);
@@ -473,10 +450,28 @@ gst_plugin_load_file (const gchar * filename, GError ** error)
     g_set_error (error,
         GST_PLUGIN_ERROR,
         GST_PLUGIN_ERROR_MODULE,
-        "gst_plugin_register_func failed for plugin \"%s\"", filename);
+        "File \"%s\" appears to be a GStreamer plugin, but it failed to initialize",
+        filename);
     g_module_close (module);
-    if (free_plugin)
-      g_free (plugin);
+    goto return_error;
+  }
+
+  /* remove signal handler */
+  _gst_plugin_fault_handler_restore ();
+  _gst_plugin_fault_handler_filename = NULL;
+  GST_INFO ("plugin \"%s\" loaded", plugin->filename);
+
+  gst_object_ref (plugin);
+  gst_default_registry_add_plugin (plugin);
+
+  g_static_mutex_unlock (&gst_plugin_loading_mutex);
+  return plugin;
+
+return_error:
+  {
+    if (plugin)
+      gst_object_unref (plugin);
+    g_static_mutex_unlock (&gst_plugin_loading_mutex);
     return NULL;
   }
 }
@@ -486,23 +481,16 @@ gst_plugin_desc_copy (GstPluginDesc * dest, const GstPluginDesc * src)
 {
   dest->major_version = src->major_version;
   dest->minor_version = src->minor_version;
-  g_free (dest->name);
   dest->name = g_strdup (src->name);
-  g_free (dest->description);
   dest->description = g_strdup (src->description);
   dest->plugin_init = src->plugin_init;
-  dest->plugin_exit = src->plugin_exit;
-  g_free (dest->version);
   dest->version = g_strdup (src->version);
-  g_free (dest->license);
   dest->license = g_strdup (src->license);
-  g_free (dest->package);
+  dest->source = g_strdup (src->source);
   dest->package = g_strdup (src->package);
-  g_free (dest->origin);
   dest->origin = g_strdup (src->origin);
 }
 
-#if 0
 /* unused */
 static void
 gst_plugin_desc_free (GstPluginDesc * desc)
@@ -511,39 +499,12 @@ gst_plugin_desc_free (GstPluginDesc * desc)
   g_free (desc->description);
   g_free (desc->version);
   g_free (desc->license);
+  g_free (desc->source);
   g_free (desc->package);
   g_free (desc->origin);
 
   memset (desc, 0, sizeof (GstPluginDesc));
 }
-#endif
-/**
- * gst_plugin_unload_plugin:
- * @plugin: The plugin to unload
- *
- * Unload the given plugin.
- *
- * Returns: whether or not the plugin unloaded
- */
-gboolean
-gst_plugin_unload_plugin (GstPlugin * plugin)
-{
-  g_return_val_if_fail (plugin != NULL, FALSE);
-
-  if (!plugin->module)
-    return TRUE;
-
-  if (g_module_close (plugin->module)) {
-    plugin->module = NULL;
-    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "plugin \"%s\" unloaded",
-        plugin->filename);
-    return TRUE;
-  } else {
-    GST_CAT_INFO (GST_CAT_PLUGIN_LOADING, "failed to unload plugin \"%s\"",
-        plugin->filename);
-    return FALSE;
-  }
-}
 
 /**
  * gst_plugin_get_name:
@@ -625,6 +586,22 @@ gst_plugin_get_license (GstPlugin * plugin)
   return plugin->desc.license;
 }
 
+/**
+ * gst_plugin_get_source:
+ * @plugin: plugin to get the source of
+ *
+ * get the source module the plugin belongs to.
+ *
+ * Returns: the source of the plugin
+ */
+G_CONST_RETURN gchar *
+gst_plugin_get_source (GstPlugin * plugin)
+{
+  g_return_val_if_fail (plugin != NULL, NULL);
+
+  return plugin->desc.source;
+}
+
 /**
  * gst_plugin_get_package:
  * @plugin: plugin to get the package of
@@ -661,10 +638,10 @@ gst_plugin_get_origin (GstPlugin * plugin)
  * gst_plugin_get_module:
  * @plugin: plugin to query
  *
- * Gets the #GModule of the plugin. If the plugin isn't loaded yet, NULL is 
+ * Gets the #GModule of the plugin. If the plugin isn't loaded yet, NULL is
  * returned.
  *
- * Returns: module belonging to the plugin or NULL if the plugin isn't 
+ * Returns: module belonging to the plugin or NULL if the plugin isn't
  *          loaded yet.
  */
 GModule *
@@ -688,9 +665,10 @@ gst_plugin_is_loaded (GstPlugin * plugin)
 {
   g_return_val_if_fail (plugin != NULL, FALSE);
 
-  return (plugin->module != NULL);
+  return (plugin->module != NULL || plugin->filename == NULL);
 }
 
+#if 0
 /**
  * gst_plugin_feature_list:
  * @plugin: plugin to query
@@ -699,7 +677,7 @@ gst_plugin_is_loaded (GstPlugin * plugin)
  * @user_data: user data passed to the filter function
  *
  * Runs a filter against all plugin features and returns a GList with
- * the results. If the first flag is set, only the first match is 
+ * the results. If the first flag is set, only the first match is
  * returned (as a list with a single object).
  *
  * Returns: a GList of features, g_list_free after use.
@@ -708,8 +686,16 @@ GList *
 gst_plugin_feature_filter (GstPlugin * plugin,
     GstPluginFeatureFilter filter, gboolean first, gpointer user_data)
 {
-  return gst_filter_run (plugin->features, (GstFilterFunc) filter, first,
+  GList *list;
+  GList *g;
+
+  list = gst_filter_run (plugin->features, (GstFilterFunc) filter, first,
       user_data);
+  for (g = list; g; g = g->next) {
+    gst_object_ref (plugin);
+  }
+
+  return list;
 }
 
 typedef struct
@@ -727,8 +713,7 @@ _feature_filter (GstPlugin * plugin, gpointer user_data)
   GList *result;
   FeatureFilterData *data = (FeatureFilterData *) user_data;
 
-  result =
-      gst_plugin_feature_filter (plugin, data->filter, data->first,
+  result = gst_plugin_feature_filter (plugin, data->filter, data->first,
       data->user_data);
   if (result) {
     data->result = g_list_concat (data->result, result);
@@ -738,15 +723,15 @@ _feature_filter (GstPlugin * plugin, gpointer user_data)
 }
 
 /**
- * gst_plugin_list_feature_list:
- * @list: a list of plugins to query
- * @filter: the filter to use
+ * gst_plugin_list_feature_filter:
+ * @list: a #GList of plugins to query
+ * @filter: the filter function to use
  * @first: only return first match
  * @user_data: user data passed to the filter function
  *
  * Runs a filter against all plugin features of the plugins in the given
- * list and returns a GList with the results. 
- * If the first flag is set, only the first match is 
+ * list and returns a GList with the results.
+ * If the first flag is set, only the first match is
  * returned (as a list with a single object).
  *
  * Returns: a GList of features, g_list_free after use.
@@ -768,6 +753,7 @@ gst_plugin_list_feature_filter (GList * list,
 
   return data.result;
 }
+#endif
 
 /**
  * gst_plugin_name_filter:
@@ -785,6 +771,7 @@ gst_plugin_name_filter (GstPlugin * plugin, const gchar * name)
   return (plugin->desc.name && !strcmp (plugin->desc.name, name));
 }
 
+#if 0
 /**
  * gst_plugin_find_feature:
  * @plugin: plugin to get the feature from
@@ -810,112 +797,144 @@ gst_plugin_find_feature (GstPlugin * plugin, const gchar * name, GType type)
   walk = gst_filter_run (plugin->features,
       (GstFilterFunc) gst_plugin_feature_type_name_filter, TRUE, &data);
 
-  if (walk)
+  if (walk) {
     result = GST_PLUGIN_FEATURE (walk->data);
 
+    gst_object_ref (result);
+    gst_plugin_feature_list_free (walk);
+  }
+
   return result;
 }
+#endif
 
-/**
- * gst_plugin_add_feature:
- * @plugin: plugin to add feature to
- * @feature: feature to add
- *
- * Add feature to the list of those provided by the plugin.
- * There is a separate namespace for each plugin feature type.
- * See #gst_plugin_get_feature_list
- */
-void
-gst_plugin_add_feature (GstPlugin * plugin, GstPluginFeature * feature)
+#if 0
+static gboolean
+gst_plugin_feature_name_filter (GstPluginFeature * feature, const gchar * name)
 {
-  GstPluginFeature *oldfeature;
-
-  /* FIXME 0.9: get reference counting somewhat right in here,
-   * GstPluginFeatures should probably be GstObjects that are sinked when
-   * adding them to a plugin */
-  g_return_if_fail (plugin != NULL);
-  g_return_if_fail (GST_IS_PLUGIN_FEATURE (feature));
-  g_return_if_fail (feature != NULL);
-
-  oldfeature = gst_plugin_find_feature (plugin,
-      GST_PLUGIN_FEATURE_NAME (feature), G_OBJECT_TYPE (feature));
-
-  if (oldfeature == feature) {
-    GST_WARNING ("feature %s has already been added",
-        GST_PLUGIN_FEATURE_NAME (feature));
-    /* g_object_unref (feature); */
-  } else if (oldfeature) {
-    GST_WARNING ("feature %s already present in plugin",
-        GST_PLUGIN_FEATURE_NAME (feature));
-    /* g_object_unref (feature); */
-  } else {
-    feature->manager = plugin;
-    plugin->features = g_list_prepend (plugin->features, feature);
-    plugin->numfeatures++;
-  }
+  return !strcmp (name, GST_PLUGIN_FEATURE_NAME (feature));
 }
+#endif
+
+#if 0
+/**
+ * gst_plugin_find_feature_by_name:
+ * @plugin: plugin to get the feature from
+ * @name: The name of the feature to find
+ *
+ * Find a feature of the given name in the given plugin.
+ *
+ * Returns: a GstPluginFeature or NULL if the feature was not found.
+ */
+GstPluginFeature *
+gst_plugin_find_feature_by_name (GstPlugin * plugin, const gchar * name)
+{
+  GList *walk;
+  GstPluginFeature *result = NULL;
+
+  g_return_val_if_fail (name != NULL, NULL);
+
+  walk = gst_filter_run (plugin->features,
+      (GstFilterFunc) gst_plugin_feature_name_filter, TRUE, (void *) name);
+
+  if (walk) {
+    result = GST_PLUGIN_FEATURE (walk->data);
+
+    gst_object_ref (result);
+    gst_plugin_feature_list_free (walk);
+  }
+
+  return result;
+}
+#endif
 
 /**
- * gst_plugin_get_feature_list:
- * @plugin: the plugin to get the features from
+ * gst_plugin_load_by_name:
+ * @name: name of plugin to load
  *
- * get a list of all the features that this plugin provides
+ * Load the named plugin. Refs the plugin.
  *
- * Returns: a GList of features, use g_list_free to free the list.
+ * Returns: A reference to a loaded plugin, or NULL on error.
  */
-GList *
-gst_plugin_get_feature_list (GstPlugin * plugin)
+GstPlugin *
+gst_plugin_load_by_name (const gchar * name)
 {
-  g_return_val_if_fail (plugin != NULL, NULL);
+  GstPlugin *plugin, *newplugin;
+  GError *error = NULL;
 
-  return g_list_copy (plugin->features);
+  GST_DEBUG ("looking up plugin %s in default registry", name);
+  plugin = gst_registry_find_plugin (gst_registry_get_default (), name);
+  if (plugin) {
+    GST_DEBUG ("loading plugin %s from file %s", name, plugin->filename);
+    newplugin = gst_plugin_load_file (plugin->filename, &error);
+    gst_object_unref (plugin);
+
+    if (!newplugin) {
+      GST_WARNING ("load_plugin error: %s\n", error->message);
+      g_error_free (error);
+      return NULL;
+    }
+    /* newplugin was reffed by load_file */
+    return newplugin;
+  }
+
+  GST_DEBUG ("Could not find plugin %s in registry", name);
+  return NULL;
 }
 
 /**
  * gst_plugin_load:
- * @name: name of plugin to load
+ * @plugin: plugin to load
  *
- * Load the named plugin.  
+ * Loads @plugin. Note that the *return value* is the loaded plugin; @plugin is
+ * untouched. The normal use pattern of this function goes like this:
  *
- * Returns: whether the plugin was loaded or not
+ * <programlisting>
+ * GstPlugin *loaded_plugin;
+ * loaded_plugin = gst_plugin_load (plugin);
+ * // presumably, we're no longer interested in the potentially-unloaded plugin
+ * gst_object_unref (plugin);
+ * plugin = loaded_plugin;
+ * </programlisting>
+ *
+ * Returns: A reference to a loaded plugin, or NULL on error.
  */
-gboolean
-gst_plugin_load (const gchar * name)
+GstPlugin *
+gst_plugin_load (GstPlugin * plugin)
 {
-  GstPlugin *plugin;
   GError *error = NULL;
+  GstPlugin *newplugin;
 
-  plugin = gst_registry_pool_find_plugin (name);
-  if (plugin) {
-    plugin = gst_plugin_load_file (plugin->filename, &error);
-    if (!plugin) {
-      GST_WARNING ("load_plugin error: %s\n", error->message);
-      g_error_free (error);
-      return FALSE;
-    }
-    return TRUE;;
+  if (gst_plugin_is_loaded (plugin)) {
+    return plugin;
   }
 
-  GST_DEBUG ("Could not find %s in registry pool", name);
-  return FALSE;
+  if (!(newplugin = gst_plugin_load_file (plugin->filename, &error)))
+    goto load_error;
+
+  return newplugin;
+
+load_error:
+  {
+    GST_WARNING ("load_plugin error: %s\n", error->message);
+    g_error_free (error);
+    return NULL;
+  }
 }
 
 /**
- * gst_library_load:
- * @name: name of library to load
+ * gst_plugin_list_free:
+ * @list: list of #GstPlugin
  *
- * Load the named library.  Name should be given as
- * &quot;liblibrary.so&quot;. (exception to this rule is 'riff', which .so name is 'gstriff')
- *
- * Returns: whether the library was loaded or not (and returns TRUE if it was already loaded)
+ * Unrefs each member of @list, then frees the list.
  */
-gboolean
-gst_library_load (const gchar * name)
+void
+gst_plugin_list_free (GList * list)
 {
-  gboolean res;
+  GList *g;
 
-  /* for now this is the same */
-  res = gst_plugin_load (name);
-
-  return res;
+  for (g = list; g; g = g->next) {
+    gst_object_unref (GST_PLUGIN (g->data));
+  }
+  g_list_free (list);
 }
