@@ -33,21 +33,27 @@
  *   <listitem><para>live sources</para></listitem>
  * </itemizedlist>
  *
- * The source can be configured to operate in a any #GstFormat with the
- * gst_base_src_set_format(). This format determines the format of the
- * internal #GstSegment and the #GST_EVENT_NEW_SEGMENT. The default format for
- * #GstBaseSrc is GST_FORMAT_BYTES.
- *
- * #GstBaseSrc always supports the push mode scheduling. If the following
+ * <refsect2>
+ * <para>
+ * The source can be configured to operate in any #GstFormat with the
+ * gst_base_src_set_format() method. The currently set format determines 
+ * the format of the internal #GstSegment and any #GST_EVENT_NEW_SEGMENT 
+ * events. The default format for #GstBaseSrc is #GST_FORMAT_BYTES.
+ * </para>
+ * <para>
+ * #GstBaseSrc always supports push mode scheduling. If the following
  * conditions are met, it also supports pull mode scheduling:
  * <itemizedlist>
- *   <listitem><para>the format is set to GST_FORMAT_BYTES (default).</para></listitem>
- *   <listitem><para>#GstBaseSrc::is_seekable returns TRUE.</para></listitem>
+ *   <listitem><para>The format is set to GST_FORMAT_BYTES (default).</para>
+ *   </listitem>
+ *   <listitem><para>#GstBaseSrc::is_seekable returns TRUE.</para>
+ *   </listitem>
  * </itemizedlist>
- *
+ * </para>
+ * <para>
  * If all the conditions are met for operating in pull mode, #GstBaseSrc is
- * automatically seekable in push mode as well. The following conditions must be
- * met to make the element seekable in push mode when the format is not
+ * automatically seekable in push mode as well. The following conditions must 
+ * be met to make the element seekable in push mode when the format is not
  * GST_FORMAT_BYTES:
  * <itemizedlist>
  *   <listitem><para>
@@ -61,34 +67,43 @@
  *     #GstBaseSrc::do_seek is implemented, performs the seek and returns TRUE.
  *   </para></listitem>
  * </itemizedlist>
- *
- * When the default format is not GST_FORMAT_BYTES, the element should ignore the
- * offset and length in the #GstBaseSrc::create method. It is recommended to subclass
- * #GstPushSrc instead in this situation.
- *
- * #GstBaseSrc has support for live sources. Live sources are sources that produce
- * data at a fixed rate, such as audio or video capture. A typical live source also
- * provides a clock to export the rate at which they produce data.
+ * </para>
+ * <para>
+ * When the default format is not GST_FORMAT_BYTES, the subclass should ignore 
+ * the offset and length in the #GstBaseSrc::create method. It is recommended 
+ * to subclass #GstPushSrc instead in this situation.
+ * </para>
+ * <para>
+ * #GstBaseSrc has support for live sources. Live sources are sources that 
+ * produce data at a fixed rate, such as audio or video capture devices. A 
+ * typical live source also provides a clock to publish the rate at which 
+ * they produce data.
  * Use gst_base_src_set_live() to activate the live source mode.
- *
- * A live source does not produce data in the PAUSED state, this means that the 
+ * </para>
+ * <para>
+ * A live source does not produce data in the PAUSED state. This means that the 
  * #GstBaseSrc::create method will not be called in PAUSED but only in PLAYING.
- * To signal the pipeline that the element will not produce data, its return
+ * To signal the pipeline that the element will not produce data, the return
  * value from the READY to PAUSED state will be GST_STATE_CHANGE_NO_PREROLL.
- * 
- * A typical live source will timestamp the buffers they create with the current
- * stream time of the pipeline. This is why they can only produce data in PLAYING,
- * when the clock is actually distributed and running.
- *
- * The #GstBaseSrc::get_times method can be used to implement pseudo-live sources.
- * The base source will wait for the specified stream time returned in 
- * #GstBaseSrc::get_times before pushing out the buffer. It only makes sense to implement
- * the ::get_times function if the source is a live source.
- *
- * There is only support in GstBaseSrc for one source pad, which should be named
- * "src". A source implementation (subclass of GstBaseSrc) should install a pad
- * template in its base_init function, like so:
- *
+ * </para>
+ * <para>
+ * A typical live source will timestamp the buffers they create with the 
+ * current stream time of the pipeline. This is one reason that they can only 
+ * produce data in PLAYING, when the clock is actually distributed and running.
+ * </para>
+ * <para>
+ * The #GstBaseSrc::get_times method can be used to implement pseudo-live 
+ * sources. The base source will wait for the specified stream time returned in 
+ * #GstBaseSrc::get_times before pushing out the buffer. 
+ * It only makes sense to implement the ::get_times function if the source is 
+ * a live source.
+ * </para>
+ * <para>
+ * There is only support in GstBaseSrc for exactly one source pad, which 
+ * should be named "src". A source implementation (subclass of GstBaseSrc) 
+ * should install a pad template in its base_init function, like so:
+ * </para>
+ * <para>
  * <programlisting>
  * static void
  * my_element_base_init (gpointer g_class)
@@ -102,8 +117,46 @@
  *   gst_element_class_set_details (gstelement_class, &amp;details);
  * }
  * </programlisting>
- *
- * Last reviewed on 2005-12-18 (0.10.0)
+ * </para>
+ * <title>Controlled shutdown of live sources in applications</title>
+ * <para>
+ * Applications that record from a live source may want to stop recording
+ * in a controlled way, so that the recording is stopped, but the data
+ * already in the pipeline is processed to the end (remember that many live
+ * sources would go on recording forever otherwise). For that to happen the
+ * application needs to make the source stop recording and send an EOS
+ * event down the pipeline. The application would then wait for an
+ * EOS message posted on the pipeline's bus to know when all data has
+ * been processed and the pipeline can safely be stopped.
+ * </para>
+ * <para>
+ * Since GStreamer 0.10.3 an application may simply set the source
+ * element to NULL or READY state to make it send an EOS event downstream.
+ * The application should lock the state of the source afterwards, so that
+ * shutting down the pipeline from PLAYING doesn't temporarily start up the
+ * source element for a second time:
+ * <programlisting>
+ * ...
+ * // stop recording
+ * gst_element_set_state (audio_source, GST_STATE_NULL);
+ * gst_element_set_locked_state (audio_source, TRUE);
+ * ...
+ * </programlisting>
+ * Now the application should wait for an EOS message
+ * to be posted on the pipeline's bus. Once it has received
+ * an EOS message, it may safely shut down the entire pipeline:
+ * <programlisting>
+ * ...
+ * // everything done - shut down pipeline
+ * gst_element_set_state (pipeline, GST_STATE_NULL);
+ * gst_element_set_locked_state (audio_source, FALSE);
+ * ...
+ * </programlisting>
+ * </para>
+ * <para>
+ * Last reviewed on 2006-03-31 (0.10.5)
+ * </para>
+ * </refsect2>
  */
 
 #include <stdlib.h>
@@ -151,6 +204,16 @@ enum
   PROP_TYPEFIND,
 };
 
+#define GST_BASE_SRC_GET_PRIVATE(obj)  \
+   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_BASE_SRC, GstBaseSrcPrivate))
+
+struct _GstBaseSrcPrivate
+{
+  gboolean last_sent_eos;       /* last thing we did was send an EOS (we set this
+                                 * to avoid the sending of two EOS in some cases) */
+  gboolean discont;
+};
+
 static GstElementClass *parent_class = NULL;
 
 static void gst_base_src_base_init (gpointer g_class);
@@ -210,7 +273,8 @@ static GstStateChangeReturn gst_base_src_change_state (GstElement * element,
     GstStateChange transition);
 
 static void gst_base_src_loop (GstPad * pad);
-static gboolean gst_base_src_check_get_range (GstPad * pad);
+static gboolean gst_base_src_pad_check_get_range (GstPad * pad);
+static gboolean gst_base_src_default_check_get_range (GstBaseSrc * bsrc);
 static GstFlowReturn gst_base_src_pad_get_range (GstPad * pad, guint64 offset,
     guint length, GstBuffer ** buf);
 static GstFlowReturn gst_base_src_get_range (GstBaseSrc * src, guint64 offset,
@@ -231,7 +295,9 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  g_type_class_add_private (klass, sizeof (GstBaseSrcPrivate));
+
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_base_src_finalize);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_base_src_set_property);
@@ -258,6 +324,8 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   klass->event = GST_DEBUG_FUNCPTR (gst_base_src_default_event);
   klass->do_seek = GST_DEBUG_FUNCPTR (gst_base_src_default_do_seek);
   klass->query = GST_DEBUG_FUNCPTR (gst_base_src_default_query);
+  klass->check_get_range =
+      GST_DEBUG_FUNCPTR (gst_base_src_default_check_get_range);
 }
 
 static void
@@ -266,6 +334,8 @@ gst_base_src_init (GstBaseSrc * basesrc, gpointer g_class)
   GstPad *pad;
   GstPadTemplate *pad_template;
 
+  basesrc->priv = GST_BASE_SRC_GET_PRIVATE (basesrc);
+
   basesrc->is_live = FALSE;
   basesrc->live_lock = g_mutex_new ();
   basesrc->live_cond = g_cond_new ();
@@ -291,7 +361,7 @@ gst_base_src_init (GstBaseSrc * basesrc, gpointer g_class)
       GST_DEBUG_FUNCPTR (gst_base_src_event_handler));
   gst_pad_set_query_function (pad, GST_DEBUG_FUNCPTR (gst_base_src_query));
   gst_pad_set_checkgetrange_function (pad,
-      GST_DEBUG_FUNCPTR (gst_base_src_check_get_range));
+      GST_DEBUG_FUNCPTR (gst_base_src_pad_check_get_range));
   gst_pad_set_getrange_function (pad,
       GST_DEBUG_FUNCPTR (gst_base_src_pad_get_range));
   gst_pad_set_getcaps_function (pad, GST_DEBUG_FUNCPTR (gst_base_src_getcaps));
@@ -317,12 +387,16 @@ static void
 gst_base_src_finalize (GObject * object)
 {
   GstBaseSrc *basesrc;
+  GstEvent **event_p;
 
   basesrc = GST_BASE_SRC (object);
 
   g_mutex_free (basesrc->live_lock);
   g_cond_free (basesrc->live_cond);
 
+  event_p = &basesrc->data.ABI.pending_seek;
+  gst_event_replace ((GstEvent **) event_p, NULL);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -785,6 +859,7 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
             src->segment.last_stop, stop, src->segment.time));
   }
 
+  src->priv->discont = TRUE;
   src->data.ABI.running = TRUE;
   /* and restart the task in case it got paused explicitely or by
    * the FLUSH_START event we pushed out. */
@@ -810,30 +885,67 @@ static gboolean
 gst_base_src_send_event (GstElement * element, GstEvent * event)
 {
   GstBaseSrc *src;
-  gboolean result;
+  gboolean result = FALSE;
 
   src = GST_BASE_SRC (element);
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_START:
+    case GST_EVENT_FLUSH_STOP:
+      break;
+    case GST_EVENT_EOS:
+    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_TAG:
+    case GST_EVENT_BUFFERSIZE:
+      break;
+    case GST_EVENT_QOS:
+      break;
     case GST_EVENT_SEEK:
     {
-      GST_OBJECT_LOCK (src);
-      /* gst_event_replace? */
-      if (src->data.ABI.pending_seek)
-        gst_event_unref (src->data.ABI.pending_seek);
-      gst_event_ref (event);
-      src->data.ABI.pending_seek = event;
-      GST_OBJECT_UNLOCK (src);
-      result = TRUE;
+      gboolean started;
+
+      GST_OBJECT_LOCK (src->srcpad);
+      if (GST_PAD_ACTIVATE_MODE (src->srcpad) == GST_ACTIVATE_PULL)
+        goto wrong_mode;
+      started = GST_PAD_ACTIVATE_MODE (src->srcpad) == GST_ACTIVATE_PUSH;
+      GST_OBJECT_UNLOCK (src->srcpad);
+
+      if (started) {
+        /* when we are running in push mode, we can execute the
+         * seek right now, we need to unlock. */
+        result = gst_base_src_perform_seek (src, event, TRUE);
+      } else {
+        GstEvent **event_p;
+
+        /* else we store the event and execute the seek when we
+         * get activated */
+        GST_OBJECT_LOCK (src);
+        event_p = &src->data.ABI.pending_seek;
+        gst_event_replace ((GstEvent **) event_p, event);
+        GST_OBJECT_UNLOCK (src);
+        /* assume the seek will work */
+        result = TRUE;
+      }
       break;
     }
+    case GST_EVENT_NAVIGATION:
+      break;
     default:
-      result = FALSE;
       break;
   }
+done:
   gst_event_unref (event);
 
   return result;
+
+  /* ERRORS */
+wrong_mode:
+  {
+    GST_DEBUG_OBJECT (src, "cannot perform seek when operating in pull mode");
+    GST_OBJECT_UNLOCK (src->srcpad);
+    result = FALSE;
+    goto done;
+  }
 }
 
 static gboolean
@@ -1123,6 +1235,10 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
       src->num_buffers_left--;
   }
 
+  GST_DEBUG_OBJECT (src,
+      "calling create offset %" G_GUINT64_FORMAT " %" G_GINT64_FORMAT, offset,
+      src->segment.time);
+
   ret = bclass->create (src, offset, length, buf);
   if (ret != GST_FLOW_OK)
     goto done;
@@ -1199,14 +1315,37 @@ gst_base_src_pad_get_range (GstPad * pad, guint64 offset, guint length,
 }
 
 static gboolean
-gst_base_src_check_get_range (GstPad * pad)
+gst_base_src_pad_check_get_range (GstPad * pad)
 {
+  GstBaseSrcClass *bclass;
   GstBaseSrc *src;
   gboolean res;
 
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
+  bclass = GST_BASE_SRC_GET_CLASS (src);
+
+  if (bclass->check_get_range == NULL) {
+    GST_WARNING_OBJECT (src, "no check_get_range function set");
+    return FALSE;
+  }
+
+  res = bclass->check_get_range (src);
+  GST_LOG_OBJECT (src, "%s() returned %d",
+      GST_DEBUG_FUNCPTR_NAME (bclass->check_get_range), (gint) res);
+
+  gst_object_unref (src);
+
+  return res;
+}
+
+static gboolean
+gst_base_src_default_check_get_range (GstBaseSrc * src)
+{
+  gboolean res;
+
   if (!GST_OBJECT_FLAG_IS_SET (src, GST_BASE_SRC_STARTED)) {
+    GST_LOG_OBJECT (src, "doing start/stop to check get_range support");
     gst_base_src_start (src);
     gst_base_src_stop (src);
   }
@@ -1216,8 +1355,6 @@ gst_base_src_check_get_range (GstPad * pad)
    * flag and is set in the _start() method. */
   res = src->random_access;
 
-  gst_object_unref (src);
-
   return res;
 }
 
@@ -1228,9 +1365,14 @@ gst_base_src_loop (GstPad * pad)
   GstBuffer *buf = NULL;
   GstFlowReturn ret;
   gint64 position;
+  gboolean eos;
+
+  eos = FALSE;
 
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
+  src->priv->last_sent_eos = FALSE;
+
   /* if we operate in bytes, we can calculate an offset */
   if (src->segment.format == GST_FORMAT_BYTES)
     position = src->segment.last_stop;
@@ -1241,8 +1383,11 @@ gst_base_src_loop (GstPad * pad)
   if (G_UNLIKELY (ret != GST_FLOW_OK)) {
     if (ret == GST_FLOW_UNEXPECTED)
       goto eos;
-    else
+    else {
+      GST_INFO_OBJECT (src, "pausing after gst_base_src_get_range() =  %d",
+          ret);
       goto pause;
+    }
   }
   if (G_UNLIKELY (buf == NULL))
     goto error;
@@ -1275,12 +1420,30 @@ gst_base_src_loop (GstPad * pad)
       position = -1;
       break;
   }
-  if (position != -1)
+  if (position != -1) {
+    if (src->segment.stop != -1) {
+      if (position >= src->segment.stop) {
+        eos = TRUE;
+        position = src->segment.stop;
+      }
+    }
     gst_segment_set_last_stop (&src->segment, src->segment.format, position);
+  }
+
+  if (G_UNLIKELY (src->priv->discont)) {
+    buf = gst_buffer_make_metadata_writable (buf);
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+    src->priv->discont = FALSE;
+  }
 
   ret = gst_pad_push (pad, buf);
-  if (G_UNLIKELY (ret != GST_FLOW_OK))
+  if (G_UNLIKELY (ret != GST_FLOW_OK)) {
+    GST_INFO_OBJECT (src, "pausing after gst_pad_push() =  %d", ret);
     goto pause;
+  }
+
+  if (eos)
+    goto eos;
 
 done:
   gst_object_unref (src);
@@ -1299,6 +1462,7 @@ eos:
               src->segment.format, src->segment.last_stop));
     } else {
       gst_pad_push_event (pad, gst_event_new_eos ());
+      src->priv->last_sent_eos = TRUE;
     }
     goto done;
   }
@@ -1314,6 +1478,7 @@ pause:
           (_("Internal data flow error.")),
           ("streaming task paused, reason %s", reason));
       gst_pad_push_event (pad, gst_event_new_eos ());
+      src->priv->last_sent_eos = TRUE;
     }
     goto done;
   }
@@ -1325,6 +1490,7 @@ error:
     src->data.ABI.running = FALSE;
     gst_pad_pause_task (pad);
     gst_pad_push_event (pad, gst_event_new_eos ());
+    src->priv->last_sent_eos = TRUE;
     goto done;
   }
 }
@@ -1595,6 +1761,8 @@ gst_base_src_activate_push (GstPad * pad, gboolean active)
     if (!gst_base_src_start (basesrc))
       goto error_start;
 
+    basesrc->priv->last_sent_eos = FALSE;
+
     /* do initial seek, which will start the task */
     GST_OBJECT_LOCK (basesrc);
     event = basesrc->data.ABI.pending_seek;
@@ -1649,6 +1817,9 @@ gst_base_src_activate_pull (GstPad * pad, gboolean active)
   } else {
     GST_DEBUG_OBJECT (basesrc, "Deactivating in pull mode");
 
+    /* don't send EOS when going from PAUSED => READY when in pull mode */
+    basesrc->priv->last_sent_eos = TRUE;
+
     if (!gst_base_src_stop (basesrc))
       goto error_stop;
 
@@ -1686,6 +1857,8 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
         no_preroll = TRUE;
         basesrc->live_running = FALSE;
       }
+      basesrc->priv->last_sent_eos = FALSE;
+      basesrc->priv->discont = TRUE;
       GST_LIVE_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
@@ -1715,9 +1888,21 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
       GST_LIVE_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+    {
+      GstEvent **event_p;
+
       if (!gst_base_src_stop (basesrc))
         goto error_stop;
+
+      if (!basesrc->priv->last_sent_eos) {
+        GST_DEBUG_OBJECT (basesrc, "Sending EOS event");
+        gst_pad_push_event (basesrc->srcpad, gst_event_new_eos ());
+        basesrc->priv->last_sent_eos = TRUE;
+      }
+      event_p = &basesrc->data.ABI.pending_seek;
+      gst_event_replace (event_p, NULL);
       break;
+    }
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
     default:
