@@ -28,7 +28,8 @@
  *
  * Data is queued till max_level buffers have been stored. Any subsequent
  * buffers sent to this filter will block until free space becomes available in
- * the buffer. The queue is typically used in conjunction with a thread.
+ * the buffer. The queue will create a new thread on the source pad to
+ * decouple the processing on sink and source pad.
  *
  * You can query how many buffers are queued with the level argument.
  *
@@ -373,6 +374,9 @@ gst_queue_init (GstQueue * queue)
   queue->min_threshold.buffers = 0;     /* no threshold */
   queue->min_threshold.bytes = 0;       /* no threshold */
   queue->min_threshold.time = 0;        /* no threshold */
+  queue->orig_min_threshold.buffers = 0;
+  queue->orig_min_threshold.bytes = 0;
+  queue->orig_min_threshold.time = 0;
 
   queue->leaky = GST_QUEUE_NO_LEAK;
   queue->srcresult = GST_FLOW_WRONG_STATE;
@@ -493,6 +497,9 @@ gst_queue_locked_flush (GstQueue * queue)
   queue->cur_level.buffers = 0;
   queue->cur_level.bytes = 0;
   queue->cur_level.time = 0;
+  queue->min_threshold.buffers = queue->orig_min_threshold.buffers;
+  queue->min_threshold.bytes = queue->orig_min_threshold.bytes;
+  queue->min_threshold.time = queue->orig_min_threshold.time;
 
   /* we deleted something... */
   g_cond_signal (queue->item_del);
@@ -561,10 +568,11 @@ gst_queue_handle_sink_event (GstPad * pad, GstEvent * event)
 
   GST_QUEUE_MUTEX_LOCK (queue);
   if (have_eos) {
-    /* FIXME, abusing the cur_level */
-    queue->cur_level.buffers = queue->max_size.buffers;
-    queue->cur_level.bytes = queue->max_size.bytes;
-    queue->cur_level.time = queue->max_size.time;
+    /* Zero the thresholds, this makes sure the queue is completely
+     * filled and we can read all data from the queue. */
+    queue->min_threshold.buffers = 0;
+    queue->min_threshold.bytes = 0;
+    queue->min_threshold.time = 0;
   }
   g_queue_push_tail (queue->queue, event);
   g_cond_signal (queue->item_add);
@@ -1059,12 +1067,15 @@ gst_queue_set_property (GObject * object,
       break;
     case ARG_MIN_THRESHOLD_BYTES:
       queue->min_threshold.bytes = g_value_get_uint (value);
+      queue->orig_min_threshold.bytes = queue->min_threshold.bytes;
       break;
     case ARG_MIN_THRESHOLD_BUFFERS:
       queue->min_threshold.buffers = g_value_get_uint (value);
+      queue->orig_min_threshold.buffers = queue->min_threshold.buffers;
       break;
     case ARG_MIN_THRESHOLD_TIME:
       queue->min_threshold.time = g_value_get_uint64 (value);
+      queue->orig_min_threshold.time = queue->min_threshold.time;
       break;
     case ARG_LEAKY:
       queue->leaky = g_value_get_enum (value);
