@@ -145,6 +145,8 @@ static gboolean
 gst_type_find_element_activate_src_pull (GstPad * pad, gboolean active);
 static GstFlowReturn
 gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind);
+static void
+gst_type_find_element_send_cached_events (GstTypeFindElement * typefind);
 
 static guint gst_type_find_element_signals[LAST_SIGNAL] = { 0 };
 
@@ -440,6 +442,7 @@ stop_typefinding (GstTypeFindElement * typefind)
                 GST_DEBUG_PAD_NAME (peer)));
         typefind->mode = MODE_ERROR;    /* make the chain function error out */
       } else {
+        gst_type_find_element_send_cached_events (typefind);
         gst_pad_push (typefind->src, typefind->store);
       }
 
@@ -470,8 +473,10 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
 
           /* we might not have started typefinding yet because there was not
            * enough data so far; just give it a shot now and see what we get */
-          caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
-              typefind->store, &prob);
+          if (typefind->store) {
+            caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
+                typefind->store, &prob);
+          }
 
           if (caps && prob >= typefind->min_probability) {
             g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
@@ -486,19 +491,16 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
           break;
         }
         default:
-          gst_event_unref (event);
+          GST_DEBUG_OBJECT (typefind, "Saving %s event to send later",
+              GST_EVENT_TYPE_NAME (event));
+          typefind->cached_events =
+              g_list_append (typefind->cached_events, event);
           res = TRUE;
           break;
       }
       break;
     case MODE_NORMAL:
-      if (FALSE) {              // GST_EVENT_TYPE (event) == GST_EVENT_DISCONTINUOUS) {
-        start_typefinding (typefind);
-        gst_event_unref (event);
-        res = TRUE;
-      } else {
-        res = gst_pad_event_default (pad, event);
-      }
+      res = gst_pad_event_default (pad, event);
       break;
     case MODE_ERROR:
       break;
@@ -508,6 +510,22 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static void
+gst_type_find_element_send_cached_events (GstTypeFindElement * typefind)
+{
+  GList *l;
+
+  for (l = typefind->cached_events; l != NULL; l = l->next) {
+    GstEvent *event = GST_EVENT (l->data);
+
+    GST_DEBUG_OBJECT (typefind, "sending cached %s event",
+        GST_EVENT_TYPE_NAME (event));
+    gst_pad_push_event (typefind->src, event);
+  }
+  g_list_free (typefind->cached_events);
+  typefind->cached_events = NULL;
+}
+
 static GstFlowReturn
 gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -516,6 +534,28 @@ gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
 
   typefind = GST_TYPE_FIND_ELEMENT (GST_PAD_PARENT (pad));
 
+  /* Shortcircuit typefinding if we already have non-any caps */
+  if (typefind->mode == MODE_TYPEFIND) {
+    const GstCaps *caps = GST_BUFFER_CAPS (buffer);
+
+    if (caps && !gst_caps_is_any (caps)) {
+      GST_DEBUG_OBJECT (typefind, "Skipping typefinding, using caps from "
+          "upstream buffer: %" GST_PTR_FORMAT, caps);
+      typefind->mode = MODE_NORMAL;
+      g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE], 0,
+          GST_TYPE_FIND_MAXIMUM, caps);
+
+      gst_type_find_element_send_cached_events (typefind);
+      if (typefind->store) {
+        GST_DEBUG_OBJECT (typefind, "Pushing store: %d",
+            GST_BUFFER_SIZE (typefind->store));
+        gst_buffer_set_caps (typefind->store, typefind->caps);
+        gst_pad_push (typefind->src, typefind->store);
+        typefind->store = NULL;
+      }
+    }
+  }
+
   switch (typefind->mode) {
     case MODE_ERROR:
       /* we should already have called GST_ELEMENT_ERROR */
@@ -724,6 +764,10 @@ gst_type_find_element_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       gst_caps_replace (&typefind->caps, NULL);
+      g_list_foreach (typefind->cached_events,
+          (GFunc) gst_mini_object_unref, NULL);
+      g_list_free (typefind->cached_events);
+      typefind->cached_events = NULL;
       break;
     default:
       break;
