@@ -914,7 +914,7 @@ print_element_list (gboolean print_all)
     gst_plugin_feature_list_free (orig_features);
   }
 
-  gst_plugin_list_free (plugins);
+  gst_plugin_list_free (orig_plugins);
 
   g_print ("\nTotal plugins: %d\nTotal features: %d\n",
       plugincount, featurecount);
@@ -1031,14 +1031,6 @@ print_element_features (const gchar * element_name)
     n_print ("%s: a typefind function\n", element_name);
     return 0;
   }
-#ifndef GST_DISABLE_URI
-  feature = gst_default_registry_find_feature (element_name,
-      GST_TYPE_URI_HANDLER);
-  if (feature) {
-    n_print ("%s: an uri handler\n", element_name);
-    return 0;
-  }
-#endif
 
   return -1;
 }
@@ -1093,8 +1085,6 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_signal_info (element);
   print_children_info (element);
 
-  gst_object_unref (factory);
-
   if (_name[0] != '\0')
     g_free (_name);
 
@@ -1147,6 +1137,7 @@ main (int argc, char *argv[])
     /* if there's a factory, print out the info */
     if (factory) {
       retval = print_element_info (factory, print_all);
+      gst_object_unref (factory);
     } else {
       retval = print_element_features (arg);
     }
